@@ -8,13 +8,6 @@ catalog = i18nCatalog("cura")
 
 def getMetaData():
     return {
-        "plugin": {
-            "name": catalog.i18nc("@label", "Removable Drive Output Device Plugin"),
-            "author": "Ultimaker B.V.",
-            "description": catalog.i18nc("@info:whatsthis", "Provides removable drive hotplugging and writing support."),
-            "version": "1.0",
-            "api": 3
-        }
     }
 
 def register(app):
