@@ -3,23 +3,41 @@
 
 import copy
 import io
+from typing import List, Optional
 import xml.etree.ElementTree as ET
 
 from UM.Resources import Resources
 from UM.Logger import Logger
-from UM.Util import parseBool
 from cura.CuraApplication import CuraApplication
 
 import UM.Dictionary
 from UM.Settings.InstanceContainer import InstanceContainer
 from UM.Settings.ContainerRegistry import ContainerRegistry
 
+
 ##  Handles serializing and deserializing material containers from an XML file
 class XmlMaterialProfile(InstanceContainer):
+    CurrentFdmMaterialVersion = "1.3"
+    Version = 1
+
     def __init__(self, container_id, *args, **kwargs):
         super().__init__(container_id, *args, **kwargs)
         self._inherited_files = []
 
+    ##  Translates the version number in the XML files to the setting_version
+    #   metadata entry.
+    #
+    #   Since the two may increment independently we need a way to say which
+    #   versions of the XML specification are compatible with our setting data
+    #   version numbers.
+    #
+    #   \param xml_version: The version number found in an XML file.
+    #   \return The corresponding setting_version.
+    def xmlVersionToSettingVersion(self, xml_version: str) -> int:
+        if xml_version == "1.3":
+            return 2
+        return 0 #Older than 1.3.
+
     def getInheritedFiles(self):
         return self._inherited_files
 
@@ -36,16 +54,14 @@ class XmlMaterialProfile(InstanceContainer):
     def setMetaDataEntry(self, key, value):
         if self.isReadOnly():
             return
-        if self.getMetaDataEntry(key, None) == value:
-            # Prevent recursion caused by for loop.
-            return
 
         super().setMetaDataEntry(key, value)
 
         basefile = self.getMetaDataEntry("base_file", self._id)  #if basefile is self.id, this is a basefile.
-        # Update all containers that share GUID and basefile
+        # Update all containers that share basefile
         for container in ContainerRegistry.getInstance().findInstanceContainers(base_file = basefile):
-            container.setMetaDataEntry(key, value)
+            if container.getMetaDataEntry(key, None) != value: # Prevent recursion
+                container.setMetaDataEntry(key, value)
 
     ##  Overridden from InstanceContainer, similar to setMetaDataEntry.
     #   without this function the setName would only set the name of the specific nozzle / material / machine combination container
@@ -91,9 +107,9 @@ class XmlMaterialProfile(InstanceContainer):
     #             container.setDirty(True)
 
     ##  Overridden from InstanceContainer
-    # base file: global settings + supported machines
+    # base file: common settings + supported machines
     # machine / variant combination: only changes for itself.
-    def serialize(self):
+    def serialize(self, ignored_metadata_keys: Optional[List] = None):
         registry = ContainerRegistry.getInstance()
 
         base_file = self.getMetaDataEntry("base_file", "")
@@ -105,12 +121,22 @@ class XmlMaterialProfile(InstanceContainer):
 
         builder = ET.TreeBuilder()
 
-        root = builder.start("fdmmaterial", { "xmlns": "http://www.ultimaker.com/material"})
+        root = builder.start("fdmmaterial",
+                             {"xmlns": "http://www.ultimaker.com/material",
+                              "version": self.CurrentFdmMaterialVersion})
 
         ## Begin Metadata Block
         builder.start("metadata")
 
         metadata = copy.deepcopy(self.getMetaData())
+        # setting_version is derived from the "version" tag in the schema, so don't serialize it into a file
+        if ignored_metadata_keys is None:
+            ignored_metadata_keys = []
+        ignored_metadata_keys = ignored_metadata_keys + ["setting_version"]
+        # remove the keys that we want to ignore in the metadata
+        for key in ignored_metadata_keys:
+            if key in metadata:
+                del metadata[key]
         properties = metadata.pop("properties", {})
 
         # Metadata properties that should not be serialized.
@@ -118,6 +144,7 @@ class XmlMaterialProfile(InstanceContainer):
         metadata.pop("variant", "")
         metadata.pop("type", "")
         metadata.pop("base_file", "")
+        metadata.pop("approximate_diameter", "")
 
         ## Begin Name Block
         builder.start("name")
@@ -143,10 +170,10 @@ class XmlMaterialProfile(InstanceContainer):
 
         for key, value in metadata.items():
             builder.start(key)
-            # Normally value is a string.
-            # Nones get handled well.
-            if isinstance(value, bool):
-                value = str(value)  # parseBool in deserialize expects 'True'.
+            if value is not None: #Nones get handled well by the builder.
+                #Otherwise the builder always expects a string.
+                #Deserialize expects the stringified version.
+                value = str(value)
             builder.data(value)
             builder.end(key)
 
@@ -202,7 +229,10 @@ class XmlMaterialProfile(InstanceContainer):
                 product = definition_id
 
             builder.start("machine")
-            builder.start("machine_identifier", { "manufacturer": definition.getMetaDataEntry("manufacturer", ""), "product":  product})
+            builder.start("machine_identifier", {
+                "manufacturer": container.getMetaDataEntry("machine_manufacturer", definition.getMetaDataEntry("manufacturer", "Unknown")),
+                "product":  product
+            })
             builder.end("machine_identifier")
 
             for instance in container.findInstances():
@@ -369,22 +399,53 @@ class XmlMaterialProfile(InstanceContainer):
         self._dirty = False
         self._path = ""
 
+    def getConfigurationTypeFromSerialized(self, serialized: str) -> Optional[str]:
+        return "materials"
+
+    def getVersionFromSerialized(self, serialized: str) -> Optional[int]:
+        data = ET.fromstring(serialized)
+
+        version = 1
+        # get setting version
+        if "version" in data.attrib:
+            setting_version = self.xmlVersionToSettingVersion(data.attrib["version"])
+        else:
+            setting_version = self.xmlVersionToSettingVersion("1.2")
+
+        return version * 1000000 + setting_version
+
     ##  Overridden from InstanceContainer
     def deserialize(self, serialized):
-        data = ET.fromstring(serialized)
+        # update the serialized data first
+        from UM.Settings.Interfaces import ContainerInterface
+        serialized = ContainerInterface.deserialize(self, serialized)
+
+        try:
+            data = ET.fromstring(serialized)
+        except:
+            Logger.logException("e", "An exception occured while parsing the material profile")
+            return
 
         # Reset previous metadata
         self.clearData() # Ensure any previous data is gone.
         meta_data = {}
         meta_data["type"] = "material"
         meta_data["base_file"] = self.id
-        meta_data["status"] = "unknown"  # TODO: Add material verfication
+        meta_data["status"] = "unknown"  # TODO: Add material verification
+
+        common_setting_values = {}
 
         inherits = data.find("./um:inherits", self.__namespaces)
         if inherits is not None:
             inherited = self._resolveInheritance(inherits.text)
             data = self._mergeXML(inherited, data)
 
+        # set setting_version in metadata
+        if "version" in data.attrib:
+            meta_data["setting_version"] = self.xmlVersionToSettingVersion(data.attrib["version"])
+        else:
+            meta_data["setting_version"] = self.xmlVersionToSettingVersion("1.2") #1.2 and lower didn't have that version number there yet.
+
         metadata = data.iterfind("./um:metadata/*", self.__namespaces)
         for entry in metadata:
             tag_name = _tag_without_namespace(entry)
@@ -403,12 +464,20 @@ class XmlMaterialProfile(InstanceContainer):
                 meta_data["material"] = material.text
                 meta_data["color_name"] = color.text
                 continue
+
+            # setting_version is derived from the "version" tag in the schema earlier, so don't set it here
+            if tag_name == "setting_version":
+                continue
+
             meta_data[tag_name] = entry.text
 
-        if not "description" in meta_data:
+            if tag_name in self.__material_metadata_setting_map:
+                common_setting_values[self.__material_metadata_setting_map[tag_name]] = entry.text
+
+        if "description" not in meta_data:
             meta_data["description"] = ""
 
-        if not "adhesion_info" in meta_data:
+        if "adhesion_info" not in meta_data:
             meta_data["adhesion_info"] = ""
 
         property_values = {}
@@ -417,46 +486,47 @@ class XmlMaterialProfile(InstanceContainer):
             tag_name = _tag_without_namespace(entry)
             property_values[tag_name] = entry.text
 
-        diameter = float(property_values.get("diameter", 2.85)) # In mm
-        density = float(property_values.get("density", 1.3)) # In g/cm3
+            if tag_name in self.__material_properties_setting_map:
+                common_setting_values[self.__material_properties_setting_map[tag_name]] = entry.text
+
+        meta_data["approximate_diameter"] = str(round(float(property_values.get("diameter", 2.85)))) # In mm
         meta_data["properties"] = property_values
 
         self.setDefinition(ContainerRegistry.getInstance().findDefinitionContainers(id = "fdmprinter")[0])
 
-        global_compatibility = True
-        global_setting_values = {}
+        common_compatibility = True
         settings = data.iterfind("./um:settings/um:setting", self.__namespaces)
         for entry in settings:
             key = entry.get("key")
-            if key in self.__material_property_setting_map:
-                global_setting_values[self.__material_property_setting_map[key]] = entry.text
+            if key in self.__material_settings_setting_map:
+                common_setting_values[self.__material_settings_setting_map[key]] = entry.text
             elif key in self.__unmapped_settings:
                 if key == "hardware compatible":
-                    global_compatibility = parseBool(entry.text)
+                    common_compatibility = self._parseCompatibleValue(entry.text)
             else:
                 Logger.log("d", "Unsupported material setting %s", key)
-        self._cached_values = global_setting_values
+        self._cached_values = common_setting_values # from InstanceContainer ancestor
 
-        meta_data["compatible"] = global_compatibility
+        meta_data["compatible"] = common_compatibility
         self.setMetaData(meta_data)
         self._dirty = False
 
         machines = data.iterfind("./um:settings/um:machine", self.__namespaces)
         for machine in machines:
-            machine_compatibility = global_compatibility
+            machine_compatibility = common_compatibility
             machine_setting_values = {}
             settings = machine.iterfind("./um:setting", self.__namespaces)
             for entry in settings:
                 key = entry.get("key")
-                if key in self.__material_property_setting_map:
-                    machine_setting_values[self.__material_property_setting_map[key]] = entry.text
+                if key in self.__material_settings_setting_map:
+                    machine_setting_values[self.__material_settings_setting_map[key]] = entry.text
                 elif key in self.__unmapped_settings:
                     if key == "hardware compatible":
-                        machine_compatibility = parseBool(entry.text)
+                        machine_compatibility = self._parseCompatibleValue(entry.text)
                 else:
                     Logger.log("d", "Unsupported material setting %s", key)
 
-            cached_machine_setting_properties = global_setting_values.copy()
+            cached_machine_setting_properties = common_setting_values.copy()
             cached_machine_setting_properties.update(machine_setting_values)
 
             identifiers = machine.iterfind("./um:machine_identifier", self.__namespaces)
@@ -473,10 +543,22 @@ class XmlMaterialProfile(InstanceContainer):
 
                 definition = definitions[0]
 
+                machine_manufacturer = identifier.get("manufacturer", definition.getMetaDataEntry("manufacturer", "Unknown")) #If the XML material doesn't specify a manufacturer, use the one in the actual printer definition.
+
                 if machine_compatibility:
                     new_material_id = self.id + "_" + machine_id
 
-                    new_material = XmlMaterialProfile(new_material_id)
+                    # The child or derived material container may already exist. This can happen when a material in a
+                    # project file and the a material in Cura have the same ID.
+                    # In the case if a derived material already exists, override that material container because if
+                    # the data in the parent material has been changed, the derived ones should be updated too.
+                    found_materials = ContainerRegistry.getInstance().findInstanceContainers(id = new_material_id)
+                    is_new_material = False
+                    if found_materials:
+                        new_material = found_materials[0]
+                    else:
+                        new_material = XmlMaterialProfile(new_material_id)
+                        is_new_material = True
 
                     # Update the private directly, as we want to prevent the lookup that is done when using setName
                     new_material._name = self.getName()
@@ -484,12 +566,14 @@ class XmlMaterialProfile(InstanceContainer):
                     new_material.setDefinition(definition)
                     # Don't use setMetadata, as that overrides it for all materials with same base file
                     new_material.getMetaData()["compatible"] = machine_compatibility
+                    new_material.getMetaData()["machine_manufacturer"] = machine_manufacturer
 
                     new_material.setCachedValues(cached_machine_setting_properties)
 
                     new_material._dirty = False
 
-                    ContainerRegistry.getInstance().addContainer(new_material)
+                    if is_new_material:
+                        ContainerRegistry.getInstance().addContainer(new_material)
 
                 hotends = machine.iterfind("./um:hotend", self.__namespaces)
                 for hotend in hotends:
@@ -503,7 +587,7 @@ class XmlMaterialProfile(InstanceContainer):
                         variant_containers = ContainerRegistry.getInstance().findInstanceContainers(definition = definition.id, name = hotend_id)
 
                     if not variant_containers:
-                        Logger.log("d", "No variants found with ID or name %s for machine %s", hotend_id, definition.id)
+                        #Logger.log("d", "No variants found with ID or name %s for machine %s", hotend_id, definition.id)
                         continue
 
                     hotend_compatibility = machine_compatibility
@@ -511,17 +595,25 @@ class XmlMaterialProfile(InstanceContainer):
                     settings = hotend.iterfind("./um:setting", self.__namespaces)
                     for entry in settings:
                         key = entry.get("key")
-                        if key in self.__material_property_setting_map:
-                            hotend_setting_values[self.__material_property_setting_map[key]] = entry.text
+                        if key in self.__material_settings_setting_map:
+                            hotend_setting_values[self.__material_settings_setting_map[key]] = entry.text
                         elif key in self.__unmapped_settings:
                             if key == "hardware compatible":
-                                hotend_compatibility = parseBool(entry.text)
+                                hotend_compatibility = self._parseCompatibleValue(entry.text)
                         else:
                             Logger.log("d", "Unsupported material setting %s", key)
 
                     new_hotend_id = self.id + "_" + machine_id + "_" + hotend_id.replace(" ", "_")
 
-                    new_hotend_material = XmlMaterialProfile(new_hotend_id)
+                    # Same as machine compatibility, keep the derived material containers consistent with the parent
+                    # material
+                    found_materials = ContainerRegistry.getInstance().findInstanceContainers(id = new_hotend_id)
+                    is_new_material = False
+                    if found_materials:
+                        new_hotend_material = found_materials[0]
+                    else:
+                        new_hotend_material = XmlMaterialProfile(new_hotend_id)
+                        is_new_material = True
 
                     # Update the private directly, as we want to prevent the lookup that is done when using setName
                     new_hotend_material._name = self.getName()
@@ -530,6 +622,7 @@ class XmlMaterialProfile(InstanceContainer):
                     new_hotend_material.addMetaDataEntry("variant", variant_containers[0].id)
                     # Don't use setMetadata, as that overrides it for all materials with same base file
                     new_hotend_material.getMetaData()["compatible"] = hotend_compatibility
+                    new_hotend_material.getMetaData()["machine_manufacturer"] = machine_manufacturer
 
                     cached_hotend_setting_properties = cached_machine_setting_properties.copy()
                     cached_hotend_setting_properties.update(hotend_setting_values)
@@ -538,11 +631,12 @@ class XmlMaterialProfile(InstanceContainer):
 
                     new_hotend_material._dirty = False
 
-                    ContainerRegistry.getInstance().addContainer(new_hotend_material)
+                    if is_new_material:
+                        ContainerRegistry.getInstance().addContainer(new_hotend_material)
 
     def _addSettingElement(self, builder, instance):
         try:
-            key = UM.Dictionary.findKey(self.__material_property_setting_map, instance.definition.key)
+            key = UM.Dictionary.findKey(self.__material_settings_setting_map, instance.definition.key)
         except ValueError:
             return
 
@@ -556,8 +650,12 @@ class XmlMaterialProfile(InstanceContainer):
         else:
             return material_name
 
+    ##  Parse the value of the "material compatible" property.
+    def _parseCompatibleValue(self, value: str):
+        return value in {"yes", "unknown"}
+
     # Map XML file setting names to internal names
-    __material_property_setting_map = {
+    __material_settings_setting_map = {
         "print temperature": "default_material_print_temperature",
         "heated bed temperature": "material_bed_temperature",
         "standby temperature": "material_standby_temperature",
@@ -569,6 +667,12 @@ class XmlMaterialProfile(InstanceContainer):
     __unmapped_settings = [
         "hardware compatible"
     ]
+    __material_properties_setting_map = {
+        "diameter": "material_diameter"
+    }
+    __material_metadata_setting_map = {
+        "GUID": "material_guid"
+    }
 
     # Map XML file product names to internal ids
     # TODO: Move this to definition's metadata
@@ -581,7 +685,8 @@ class XmlMaterialProfile(InstanceContainer):
         "Ultimaker 2 Extended": "ultimaker2_extended",
         "Ultimaker 2 Extended+": "ultimaker2_extended_plus",
         "Ultimaker Original": "ultimaker_original",
-        "Ultimaker Original+": "ultimaker_original_plus"
+        "Ultimaker Original+": "ultimaker_original_plus",
+        "IMADE3D JellyBOX": "imade3d_jellybox"
     }
 
     # Map of recognised namespaces with a proper prefix.
@@ -610,4 +715,4 @@ def _indent(elem, level = 0):
 # We are only interested in the actual tag name, so discard everything
 # before the last }
 def _tag_without_namespace(element):
-    return element.tag[element.tag.rfind("}") + 1:]
+    return element.tag[element.tag.rfind("}") + 1:]
\ No newline at end of file
